/*	Justin Fiedler
	MyGlWindow.h

	Handles most of the Graphics in a Gl_Window that interacts with the UI of main.cpp.
	Uses interface from Cubeit.cpp and Cubeit3.cpp however vastly reordered to make it
	more comprehensiable.  This file also keeps track of all the control points, track
	points and axis data for constructing the track.

	The following functions are more or less verbatum from the Cubeit programs.
		- setProjection
		- draw
		- handle
		- doPick
		- getMouseNDC
		- drawPoints
		- xRollerCB
		- yRollerCB
		- zRollerCB
		- makeRollers
*/

#ifndef _MYGLWINDOW_
#define _MYGLWINDOW_

#include <FL/Fl_Gl_Window.h>
#include <Fl/Fl.h>
#include <Fl/Fl_Multiline_Output.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <Fl/Fl_Double_Window.h>
#include <Fl/Fl_Button.h>
#include <FL/FL_Roller.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Input.H>
#include "stdio.h"
#include "math.h"

#include "point.h"
#include "curve.h"
#include "models.h"
#include "MyArcBall.H"
#include "3DUtils.H"
#include <vector>

#include <iostream>
#include <fstream>
#include <string>
using namespace std;


//A vector of control points for curves
extern std::vector<point> cPoints;
//A vector of track points for the coaster
extern std::vector<point> tPoints;
//A vector of vectors to handle proper car rotation
extern std::vector<point> rPoints;

//the currently selected control point
extern int selectedPoint;

class MyGlWindow : public Fl_Gl_Window {
	public:
		bool animate;		//toggles animation on and off
		bool mainView;		//switches between main view and train view
		bool arcMode;		//the type of curve to construct

		bool drag;			//if user is dragging a control point

		//draw the control points
		void drawPoints(int doColors=1, int doNames=0);
		//draw the train cars
		void drawCars(int doColors=1, int doNames=0);
		//draw the trees
		void drawTrees(int doColors=1, int doNames=0);

		/* Use FlTk Rollers to move the selected cube
		 *
		 * Make rollers - Mus be inside of an FlTk Window
		 *                give an xy position and size for the rollers
		 *                also give it an fltk widget to damage (your window)
		 *				  anytime something changes
		 */
		void makeRollers(int x, int y, int h, Fl_Widget* w);
		
		//constructor
		MyGlWindow(int x, int y, int w, int h);
	private:
		//view parameters
		float eyeZ;				//viewing distance from orgin
		float fieldOfView;		//field of view
		MyArcBall ball;			//used for viewing

		void draw();					// standard FlTk
		int handle(int);				// standard FlTk

		// put things to set up the projection into a method so we can use
		// it places other than draw
		// since we might want to put the projection onto the stack after a
		// pick matrix, provide an option to not clear the projection stack
		void setProjection(int clearProjection = 1);

		// pick a control point
		void doPick();

		// internal routine
		// this gets the position of the mouse in "normalized device coordinates"
		// that is X & Y go from -1 to 1 (with zero at the center)
		void getMouseNDC(float& x, float& y);
};

#endif