/*	Justin Fiedler
	MyVector.cpp

	A simple 3D vector class
*/

#include "MyVector.h"

/*set the point to 3d coordinates
*/
void MyVector::set3d(double _x, double _y, double _z){
	x = _x;
	y = _y;
	z = _z;
}

/*set the point to another point
*/
void MyVector::setv(MyVector _v){
	x = _v.x;
	y = _v.y;
	z = _v.z;
	w = _v.w;
}


/*normalize the vector to magnitude <magnitude>
*/
void MyVector::normalize(double magnitude){
	double sum = sqrt(x*x + y*y + z*z)/magnitude;
	x /= sum;
	y /= sum;
	z /= sum;
}

/*multiplies vector by scalar
*/
MyVector MyVector::scalarMult(double s){
	MyVector v;
	v.set3d(x*s, y*s, z*s);
	return v;
}

/*divides vector by scalar
*/
MyVector MyVector::scalarDiv(double s){
	MyVector v;
	v.set3d(x/s, y/s, z/s);
	return v;
}

/*returns the magnitude of a cartesian vector
*/
double MyVector::mag(){
	return sqrt(x*x + y*y + z*z);
}

/* converts vector from Cartesian to Polar
*/
MyVector MyVector::toPolar(){
	MyVector p;
	p.set3d(sqrt(x*x + y*y), atan2(x, y) , 0);
	return p;
}

/* converts vector from Polar to Cartesian
*/
MyVector MyVector::toCartesian(){
	MyVector c;
	c.set3d(x*cos(y), x*sin(y), 0);
	return c;
}

/*returns the cross product of the two vectors
*/
MyVector MyVector::cross(MyVector _v){
	MyVector c;
	c.set3d(y*_v.z - z*_v.y, z*_v.x - x*_v.z, x*_v.y - y*_v.x);
	return c;
}

/*returns the dot product of the two vectors
*/
double MyVector::dot(MyVector _v){
	return x*_v.x + y*_v.y + z*_v.z;
}

double degToRad(double deg){
	return deg/180*M_PI;
}

double radToDeg(double rad){
	return rad*180/M_PI;
}