/*	Justin Fiedler
	MyVector.h

	A simple 3D vector class
*/

#ifndef _MY_VECTOR_
#define _MY_VECTOR_

#include <math.h>

static double M_PI = 3.14159265358979323846;

class MyVector {
	public:
		double x,y,z,w;
		
		//constructors
		inline MyVector(double _x=0, double _y=0, double _z=0, double _w=0) 
				:	x(_x), y(_y), z(_z), w(_w) {}
		inline MyVector(const MyVector& v)
				:	x(v.x), y(v.y), z(v.z), w(v.w){}

		/*set the vector to 3d coordinates
		*/
		void set3d(double _x, double _y, double _z);
		/*set the vector to another point
		*/
		void setv(MyVector _v);
		/*normalize the vector to magnitude <magnitude>
		*/

		void normalize(double magnitude=1);
		/*multiplies vector by scalar
		*/
		MyVector scalarMult(double s);

		/*divides vector by scalar
		*/
		MyVector scalarDiv(double s);

		/*returns the magnitude of a cartesian vector
		*/
		double mag();

		/* converts vector from Cartesian to Polar
		*/
		MyVector toPolar();

		/* converts vector from Polar to Cartesian
		*/
		MyVector toCartesian();

		/*returns the cross product of the two vectors
		*/
		MyVector cross(MyVector _v);
		
		/*returns the dot product of the two vectors
		*/
		double dot(MyVector _v);
};

double degToRad(double deg);
double radToDeg(double rad);

#endif
