#include "cCurve.h"

/*constructors*/
cCurve::cCurve(){
	setTension(0);
}

cCurve::cCurve(std::vector<CPoint> _cPoints){
	setTension(0);
	for(int i = 0; i < _cPoints.size(); i++){
		cPoints.push_back(_cPoints[i]);
	}
}

/*set the basis matrix to the selected curve type <cType> of tension <t>
*/
void cCurve::setTension(float tension){
	float s = (float)(1 - tension) / 2.0f;
	B[4] = -s;
	B[6] = s;
	B[8] = 2 * s;
	B[9] = s - 3;
	B[10] = 3 - (2 * s);
	B[11] = -s;
	B[12] = -s;
	B[13] = 2 - s;
	B[14] = s - 2;
	B[15] = s;
}

/*blending function computation (used to simplify value computation)
*/
float cCurve::compFromMatrix(float u, float p0, float p1, float p2, float p3)
{	float x, y, z, w;

	x = (B[0]  * p0) + (B[1]  * p1) + (B[2]  * p2) + (B[3]  * p3);
	y = (B[4]  * p0) + (B[5]  * p1) + (B[6]  * p2) + (B[7]  * p3);
	z = (B[8]  * p0) + (B[9]  * p1) + (B[10] * p2) + (B[11] * p3);
	w = (B[12] * p0) + (B[13] * p1) + (B[14] * p2) + (B[15] * p3);

	return ((1 * x) + (u * y) + (pow(u, 2) * z) + (pow(u, 3) * w));
}

/*return the point on the curve at "time" <u>
*/
MyVector cCurve::getPosition(float u, MyVector& p0, MyVector& p1, MyVector& p2, MyVector& p3)
{
	MyVector out;

	out.x = compFromMatrix(u, p0.x, p1.x, p2.x, p3.x);
	out.y = compFromMatrix(u, p0.y, p1.y, p2.y, p3.y);
	out.z = compFromMatrix(u, p0.z, p1.z, p2.z, p3.z);
	out.w = compFromMatrix(u, p0.w, p1.w, p2.w, p3.w);

	return out;
}

/*derivative of curve at <u>
*/
MyVector cCurve::getDerivative(float u, MyVector& p0, MyVector& p1, MyVector& p2, MyVector& p3)
{
	MyVector vec, vec2;

	vec = getPosition(u, p0, p1, p2, p3);
	vec2 = getPosition(u + .01f, p0, p1, p2, p3);
	vec2.x -= vec.x;
	vec2.y -= vec.y;
	vec2.z -= vec.z;

	return vec2;
}

/*add control points to curve
*/
void cCurve::addPoints(std::vector<CPoint> points){
	for(int i = 0; i < points.size(); i++){
		cPoints.push_back(points[i]);
	}
}

/*add control points to curve
*/
void cCurve::popCPoint(){
	cPoints.pop_back();
}
		