#include "curve.h"
#include <iostream>
using namespace std;

/*the basis function of the current curve type (4x4 matrix)
*/
float *B = new float[16];

/*tension function for cardinal cubics
*/
float s(float t){
	return .5*(1 - t);
}

/*returns 1D coordinate of the 2D coodinate (this makes matrix access more logical)
*/
int get1DPos(int r, int c, int numcol) {
	return (r * numcol) + c;
}

/*returns 1D coordinate of the 2D coodinate (this makes matrix access more logical)
	this is tuned for a 4x4 matrix 
*/
int bPos(int r, int c){
	return get1DPos(r, c, 4);
}

/*set the basis matrix to the selected curve type <cType> of tension <t>
*/
void setB(int cType, float t){
	float Hermite[16] ={  1,  0,  0,  0,
						 -3, -2,  3, -1,						 
						  0,  1,  0,  0,
						  2,  1, -2,  1};

	float BSpline[16] ={  1,  4,  1,  0,
						 -3,  0,  3,  0,
						  3, -6,  3,  0,
						 -1,  3, -3,  1};
	switch(cType){
		case HERMITE:
			for(int i = 0; i < 16; i++){
				B[i] = Hermite[i];
			}
			break;
		case CARDINAL:
			B[0] = 0;
			B[1] = 1;
			B[2] = 0;
			B[3] = 0;

			B[4] = -s(t);
			B[5] = 0;
			B[6] = s(t);
			B[7] = 0;
			
			B[bPos(2, 0)] = 2*s(t);
			B[bPos(2, 1)] = s(t) - 3;
			B[bPos(2, 2)] = 3 - 2*s(t);
			B[bPos(2, 3)] = -s(t);
			
			B[bPos(3, 0)] = -s(t);
			B[bPos(3, 1)] = 2 - s(t);
			B[bPos(3, 2)] = s(t) - 2;
			B[bPos(3, 3)] = s(t);
			break;
		case BSPLINE:
			for(int i = 0; i < 16; i++){
				B[i] = (1/6.0f) * BSpline[i];
			}
			break;
	};
}

/*sets a 4x1 vector to the given floats this makes multiplication
	of the matrixes easier
*/
void setP(float *p, float p0_c, float p1_c, float p2_c, float p3_c){
	p[0] = p0_c;
	p[1] = p1_c;
	p[2] = p2_c;
	p[3] = p3_c;
}

/*calculate the f value of a single component (x,y,z)
*/
float f_c(float u, float* B, float* p){
	//calculate B*p (blending functions)
	float Bp[4];
	for(int r = 0; r < 4; r++){
		Bp[r] = B[bPos(r, 0)] * p[0] 
			  + B[bPos(r, 1)] * p[1]
			  + B[bPos(r, 2)] * p[2]
			  + B[bPos(r, 3)] * p[3];
	}
	
	//calculate and return f(u) = u*Bp
	return (Bp[0] + u * Bp[1] + u*u * Bp[2] + u*u*u * Bp[3]);
}

/*return the point on the curve at "time" <u>
*/
point f(float u, float* B, point p0, point p1, point p2, point p3){
	point fu;
	float p[4];
	
	//do X
	setP(p, p0.x, p1.x, p2.x, p3.x);
	fu.x = f_c(u, B, p);

	//do Y
	setP(p, p0.y, p1.y, p2.y, p3.y);
	fu.y = f_c(u, B, p);

	//do Z
	setP(p, p0.z, p1.z, p2.z, p3.z);
	fu.z = f_c(u, B, p);

	return fu;
}

/*return derivative at the point on the curve at "time" <u>
*/
point df(float u, float* B, point p0, point p1, point p2, point p3){
	point tmp = f(u, B, p0, p1, p2, p3);
	point tmp1 = f(u+.05f, B, p0, p1, p2, p3);

	tmp1.x -= tmp.x;
	tmp1.y -= tmp.y;
	tmp1.z -= tmp.z;

	return tmp1;
}

/*generates and saves all track points from current controls(this is slow)
*/
void genAllArcPoints(int mode){
	float u_inc = .0001f;		//u step
	float arc_d_step = .01f;	//desired arc length
	float cur_arc_d = 0;		//current arc length allong curve
	
	//clear out old points
	tPoints.clear(); rPoints.clear();
	for(int i = 0; i < cPoints.size(); i++){
		switch(mode){
			case ARC_LENGTH:
				cur_arc_d = genALPArcPoints(u_inc, arc_d_step, cur_arc_d,
									cPoints[i%cPoints.size()], cPoints[(i + 1)%cPoints.size()],
									cPoints[(i + 2)%cPoints.size()], cPoints[(i + 3)%cPoints.size()]);
				break;
			default:
				genNALPArcPoints(arc_d_step,
						cPoints[i%cPoints.size()], cPoints[(i + 1)%cPoints.size()],
						cPoints[(i + 2)%cPoints.size()], cPoints[(i + 3)%cPoints.size()]);
		};
	}
	//generate the rail points(vectors) from the track points
	generateRailPoints();
	//handle case that train is no longer on the track
	if(!(trainCars[0].curPoint < tPoints.size())){
		trainCars[0].curPoint = tPoints.size() - 1;
	}
	//set all car positions
	for(int i = 1; i < trainCars.size(); i++){
		if(trainCars[(i - 1)].curPoint >= trainCars[i - 1].length){
			trainCars[i].curPoint = trainCars[i - 1].curPoint - trainCars[i-1].length;
		} else {
			trainCars[i].curPoint = (trainCars[i - 1].curPoint - trainCars[i-1].length) + (tPoints.size()-1);
		}
	}
}

/*generates arc length parameterized track points
	of current arc function f(), from given control points <p0, p1, p2, p3>
	each <arc_d_step> apart
*/
float genALPArcPoints(float u_inc, float arc_d_step, float cur_arc_d, point p0, point p1, point p2, point p3){
	point p, np;
	p.setp(f(0, B, p0, p1, p2, p3));
	for(float u = u_inc; u < 1; u += u_inc){
		//get next point
		np.setp(f(u, B, p0, p1, p2, p3));
		
		//get current arc distance
		cur_arc_d += sqrt((p.x - np.x)*(p.x - np.x) +
						  (p.y - np.y)*(p.y - np.y) +
						  (p.z - np.z)*(p.z - np.z));

		//if current distance is >= to step distance set a track point here 
		if(cur_arc_d >= arc_d_step){
			//cout << cur_arc_d << endl;
			tPoints.push_back(np);			
			cur_arc_d -= arc_d_step;
		}
		
		//set p to next p
		p.setp(np);
	}
	return cur_arc_d;
}

/*generates non arc length parameterized track points at even u_inc
	of current arc function f(), from given control points <p0, p1, p2, p3>
*/
void genNALPArcPoints(float u_inc, point p0, point p1, point p2, point p3){
	for(float u = 0; u < 1; u += u_inc){
		tPoints.push_back(f(u, B, p0, p1, p2, p3));
	}
}

/*draws Arcs according to current track points
*/
void drawArcsp(int doColors){
	if(doColors == 0){ 
		glColor4f(0,0,0,.5);
	} else {
		glColor3f(1,1,0);
	}

	//loop over track points and connect the dots with a line
	glPushMatrix();
		glBegin(GL_LINE_LOOP);
		for(int i = 0; i < rPoints.size(); i++){
			glVertex3d(tPoints[i].x + rPoints[i].x, tPoints[i].y + rPoints[i].y, tPoints[i].z + rPoints[i].z);
		}
		glEnd();
		glBegin(GL_LINE_LOOP);
		for(int i = 0; i < rPoints.size(); i++){
			glVertex3d(tPoints[i].x - rPoints[i].x, tPoints[i].y - rPoints[i].y, tPoints[i].z - rPoints[i].z);
		}
		glEnd();
		glBegin(GL_LINES);
		for(int i = 0; i < rPoints.size(); i += 20){
			glVertex3d(tPoints[i].x + rPoints[i].x, tPoints[i].y + rPoints[i].y, tPoints[i].z + rPoints[i].z);
			glVertex3d(tPoints[i].x - rPoints[i].x, tPoints[i].y - rPoints[i].y, tPoints[i].z - rPoints[i].z);
		}
		glEnd();
	glPopMatrix();
}

/*calculate and draws the track according to current control points without 
	saving them to track points (this is fast)
*/
void drawArcc(point p0, point p1, point p2, point p3){
	float inc = .05;
	glPushMatrix();
	glBegin(GL_LINE_STRIP);
	for(float u = 0; u <= 1.0001f; u += inc){
		//get current point and connect it to last point
		point p = f(u, B, p0, p1, p2, p3);
		glVertex3d( p.x, p.y, p.z);
	}
	glEnd();
	glPopMatrix();
}

void drawArcsc(int doColors){
	if(doColors == 0) glColor4f(0,0,0,.5);
	for(int i = 0; i < cPoints.size(); i++){
		drawArcc(cPoints[i%cPoints.size()], cPoints[(i + 1)%cPoints.size()],
				  cPoints[(i + 2)%cPoints.size()], cPoints[(i + 3)%cPoints.size()]);
	}
}

/*generates rail points(really vectors) to handle dual rails
	takes cross product of normal <0,1,0> to get new x axis
	this axis then can be used to calculate the rails
*/
void generateRailPoints(){
	point x_vect, z_vect, normal;
	//up vector
	normal.set3d(0, 1, 0);
	for(int i = 0; i < tPoints.size(); i++){
		//calulate the derivate at the point
		x_vect.x = tPoints[(i + 1)%tPoints.size()].x - tPoints[i].x;
		x_vect.y = tPoints[(i + 1)%tPoints.size()].y - tPoints[i].y;
		x_vect.z = tPoints[(i + 1)%tPoints.size()].z - tPoints[i].z;

		//get a perpendicular vector <the new axis>
		z_vect.setp(x_vect.crossProduct(normal));
		//normalize to half of train scale so that train fits to track
		z_vect.normalize(trainCars[0].scale/2.0);
		rPoints.push_back(z_vect);
	}
}
		