/*	Justin Fiedler
	curve.h

	A class to construct a set of curves given a collection of control points.  Curves are made
	into a vector of track points (tPoints) that conform to the type of curve selected (cardinal
	and bspline) hermite is an option but it doesnt work and is only there for personal enjoyment
	and possibly aesethic value and should otherwise be ignored.  Also track points can be made
	to be arc length parameterized or not.  Life doesnt get much better than that.
*/

#ifndef _CURVE_
#define _CURVE_

#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "stdio.h"
#include "math.h"

#include "point.h"
#include "MyGlWindow.h"


//some constants to select the type of curve
const int HERMITE  = 0;
const int CARDINAL = 1;
const int BSPLINE  = 2;


//constants to select the type of parameterization
const int NON_ARC_LENGTH = 0;
const int ARC_LENGTH = 1;

//the basis function of the current curve type
extern float* B;
/*used to calculate tension for cardinal curves
*/
float s(float t);

/*returns 1D coordinate of the 2D coodinate (this makes matrix access more logical)
	this is tuned for a 4x4 matrix 
*/
int bPos(int x, int y);

/*set the basis matrix to the selected curve type
*/
void setB(int cType, float t=0);

/*return the point on the curve at "time" <u>
*/
point f(float u, float* B, point p0, point p1, point p2, point p3);

/*return the derivative on the curve at "time" <u>
*/
point df(float u, float* B, point p0, point p1, point p2, point p3);


/*calculate and draws the track according to current control points without 
	saving them to track points (this is fast)
*/
void drawArcc(point p0, point p1, point p2, point p3);
void drawArcsc(int doColors=1);

/*generates and saves all track points from current controls(this is slow)
*/
void genAllArcPoints(int mode=0);

/*generates arc length parameterized track points
	of current arc function f(), from given control points <p0, p1, p2, p3>
	each <arc_d_step> apart
*/
float genALPArcPoints(float u_inc, float arc_d_step, float cur_arc_d, point p0, point p1, point p2, point p3);

/*generates non arc length parameterized track points at even u_inc
	of current arc function f(), from given control points <p0, p1, p2, p3>
*/
void genNALPArcPoints(float u_inc, point p0, point p1, point p2, point p3);

/*draws Arcs according to current track points
*/
void drawArcsp(int doColors=1);

/*generates rail points(really vectors) to handle dual rails
*/
void generateRailPoints();


#endif