/*	Justin Fiedler
	main.cpp
	
	This file creates the main window with the UI and an opengl window.  It handles the callbacks
	of the buttons and sliders.

	It was build in the tradition of the OpenGL Tutortials lacks enough resemblence to really
	acredit them.
*/
#include "MyGlWindow.h"

static Fl_Input* input;
static MyGlWindow* gl_win;

//callback functions
void addButtonCallback(Fl_Widget* w);
void removeButtonCallback(Fl_Widget* w);
void animateButtonCallback(Fl_Widget* w);
void viewButtonCallback(Fl_Widget* w);

void hermiteButtonCallback(Fl_Widget* w);
void cardinalButtonCallback(Fl_Widget* w);
void bsplineButtonCallback(Fl_Widget* w);

void tensionSliderCallback(Fl_Widget *w);
void velocitySliderCallback(Fl_Widget *w);

void saveButtonCallback(Fl_Widget *w);
void loadButtonCallback(Fl_Widget *w);

void arcModeButtonCallback(Fl_Widget *w);
void velocitySliderCallback(Fl_Widget *w);
void addCarButtonCallback(Fl_Widget *w);
void removeCarButtonCallback(Fl_Widget *w);

void idleCallback(void *p);

int curveType = CARDINAL;

main()
{	//the main window
	Fl_Double_Window* wind = new Fl_Double_Window(100,100,600,450,"Rollercoaster Tycoon .001 beta");
	wind->begin();		// put widgets inside of the window
		//the opengl window
		MyGlWindow* gl = new MyGlWindow(10,10,380,380);
		gl_win = gl;
		//some instructions
		Fl_Multiline_Output* out = new Fl_Multiline_Output(420,190,580-420,380-190);
		out->value("Rollercoaster - 3D UI\n\nLeftMouse: select&drag\nRightMouse: ArcBall\n\nDrag parallel to floor\nHold CTRL for vertical\nScroll wheel zooms");
		//add a new control point
		Fl_Button* addButton = new Fl_Button(400, 10, 100, 20, "Add Point");
		addButton->callback(addButtonCallback);
		//remove the selected control point
		Fl_Button* removeButton = new Fl_Button(500, 10, 100, 20, "Remove Point");
		removeButton->callback(removeButtonCallback);
		//start the train
		Fl_Button* animateButton = new Fl_Button(400, 30, 100, 20, "Animate");
		animateButton->callback(animateButtonCallback);
		//change view
		Fl_Button* viewButton = new Fl_Button(500, 30, 100, 20, "Toggle View");
		viewButton->callback(viewButtonCallback);
		//makes hermite curves, this doesnt really work so dont use it
		Fl_Button* hermiteButton = new Fl_Button(400, 50, 66, 20, "Hermite");
		hermiteButton->callback(hermiteButtonCallback);
		//makes cardinal curves
		Fl_Button* cardinalButton = new Fl_Button(467, 50, 66, 20, "Cardinal");
		cardinalButton->callback(cardinalButtonCallback);
		//makes bspline curves
		Fl_Button* bsplineButton = new Fl_Button(533, 50, 66, 20, "BSpline");
		bsplineButton->callback(bsplineButtonCallback);

		//the name of the file to save to/load from
		Fl_Input* fileNameInput = new Fl_Input(400, 70, 200, 20);
		fileNameInput->value("u:\\private\\559\\coaster.txt");
		input = fileNameInput;

		//save to file
		Fl_Button* saveButton = new Fl_Button(500, 90, 50, 20, "Save");
		saveButton->callback(saveButtonCallback);
		//load from file
		Fl_Button* loadButton = new Fl_Button(550, 90, 50, 20, "Load");
		loadButton->callback(loadButtonCallback);
		
		//switch between arc length parameterization and non
		Fl_Button* arcModeButton = new Fl_Button(400, 110, 50, 20, "ALP");
		arcModeButton->callback(arcModeButtonCallback);
		
		//adds a car to the back of the train
		Fl_Button* addCarButton = new Fl_Button(450, 110, 50, 20, "Car++");
		addCarButton->callback(addCarButtonCallback);

		//removes a car to the back of the train
		Fl_Button* removeCarButton = new Fl_Button(500, 110, 50, 20, "Car--");
		removeCarButton->callback(removeCarButtonCallback);

		//adjust the tension for the cardinal curves
		Fl_Value_Slider* tensionSlider = new Fl_Value_Slider(400, 130, 100, 20, "Tension");
		tensionSlider->type(FL_HOR_SLIDER);
		tensionSlider->bounds(0.0, 1.0);
		tensionSlider->value(0.0);
		tensionSlider->callback(tensionSliderCallback);

		//adjusts the velocity
		Fl_Value_Slider* velocitySlider = new Fl_Value_Slider(500, 130, 100, 20, "Velocity");
		velocitySlider->type(FL_HOR_SLIDER);
		velocitySlider->bounds(1, 20);
		velocitySlider->value(1);
		velocitySlider->callback(velocitySliderCallback);

		Fl::add_idle(idleCallback, gl);
	wind->end();
	wind->show();	// this actually opens the window
	
	//add the first car onto the track
	trainCars.push_back(Train(0, .5));

	// add some control points
	cPoints.push_back(point(-3,3, -4));
	cPoints.push_back(point(0,3, 3));
	cPoints.push_back(point(4,3,3));
	cPoints.push_back(point(6,3,0));
	
	//and some shrubbery
	trees.push_back(Tree(point(4, 0, 0), 1));

	//set the curve type
	setB(curveType);
	//generate the points on the track
	genAllArcPoints(gl_win->arcMode);

	// become interactive
	Fl::run();
	delete wind;

	return 1;
}

/* if 4 or more adds a new control point on the curve between last 2 control points
	otherwise it adds a new point at 0,0,0
*/
void addButtonCallback(Fl_Widget* w){
	cout << "Point Added" << endl;
	point p;
	if(cPoints.size() >= 4) {
		int last = cPoints.size() - 1;
		p = f(.5, B, cPoints[(last - 1)], cPoints[last], cPoints[0], cPoints[1]);
	} else {
		p = point(0, 0, 0);
	}
	cPoints.push_back(p);
	genAllArcPoints(gl_win->arcMode);
	gl_win->redraw();
}

/*removes the selected control point
	doesnt allow last control point to be removed
*/
void removeButtonCallback(Fl_Widget* w){
	if(cPoints.size() > 1 && selectedPoint != -1)
	{	cPoints.erase(cPoints.begin() + selectedPoint, cPoints.begin() + selectedPoint + 1);
		genAllArcPoints(gl_win->arcMode);
		selectedPoint = -1;
		gl_win->redraw();
	}
}

/*switch animation on and off
*/
void animateButtonCallback(Fl_Widget* w){
	gl_win->animate = !gl_win->animate;
}

/*	toogle between main view and train view
*/
void viewButtonCallback(Fl_Widget* w){
	gl_win->mainView = !gl_win->mainView;
	gl_win->damage(1);
}

/*	construct hermite curves
	this doesnt really work but it looks cool
*/
void hermiteButtonCallback(Fl_Widget* w){
	setB(curveType=HERMITE);
	genAllArcPoints(gl_win->arcMode);
	gl_win->damage(1);
}

/* construct cardinal curves
*/
void cardinalButtonCallback(Fl_Widget* w){
	setB(curveType=CARDINAL);
	genAllArcPoints(gl_win->arcMode);
	gl_win->damage(1);
}

/*	construct bsplines
*/
void bsplineButtonCallback(Fl_Widget* w){
	setB(curveType=BSPLINE);
	genAllArcPoints(gl_win->arcMode);
	gl_win->damage(1);
}

/* varies the tension of the cardinal curves
*/
void tensionSliderCallback(Fl_Widget *w){
	setB(curveType, static_cast<Fl_Value_Slider*>(w)->value());
	genAllArcPoints(gl_win->arcMode);
	gl_win->damage(1);
}

/* saves control points to file
*/
void saveButtonCallback(Fl_Widget *w){
	cout << "Save: " << cPoints.size() << endl;
	//open file stream
	ofstream myfile(input->value());
	if (myfile.is_open()) {
		//record the number of points
		myfile << cPoints.size() << endl;
		//save it point
		for(int i = 0; i < cPoints.size(); i++){
			myfile << cPoints[i].x << endl << cPoints[i].y << endl << cPoints[i].z << endl;
		}
		myfile.close();
	} else {
		cout << "Unable to open file";
	}
}

/* loads control points from file
*/
void loadButtonCallback(Fl_Widget *w){
	string xs, ys, zs;
	int count;
	ifstream myfile (input->value());

	//if the file opens read the input and construct vector of control points
	if (myfile.is_open()){
		getline(myfile, xs);
		count = atoi(xs.c_str());
		cPoints.clear();
		cout << "Clear: " << cPoints.size() << endl;
		for(int i = 0; i < count; i++){
			getline(myfile, xs);
			getline(myfile, ys);
			getline(myfile, zs);
			cPoints.push_back(point(atof(xs.c_str()), atof(ys.c_str()), atof(zs.c_str())));
		}
		cout << "Build: " << cPoints.size() << endl;
		myfile.close();
	} else {
		cout << "Unable to open file";
	}
	//generate the new track points from control points
	genAllArcPoints(gl_win->arcMode);
	gl_win->damage(1);
}

/* alternates between ALP and NALP
*/
void arcModeButtonCallback(Fl_Widget *w){
	gl_win->arcMode = !gl_win->arcMode;
	if(gl_win->arcMode) {
		static_cast<Fl_Button*>(w)->label("ALP");	//non arc length parameterized
	} else {
		static_cast<Fl_Button*>(w)->label("NALP");	//arc length parameterized
	}
	genAllArcPoints(gl_win->arcMode);
	gl_win->damage(1);
}

/* varies the velocity of the train
	velocity is in trak point distances
*/
void velocitySliderCallback(Fl_Widget *w){
	//change train velocity to slider value
	for(int i = 0; i < trainCars.size(); i++){
		trainCars[i].vel = static_cast<Fl_Value_Slider*>(w)->value();
	}
}

/*  adds another train to the track behind the last car
*/
void addCarButtonCallback(Fl_Widget *w){
	//add a car 
	int curPoint;
	//create new car
	Train new_car;
	//add it at the end of the line
	if(trainCars[(trainCars.size() - 1)].curPoint >= trainCars[trainCars.size() - 1].length){
		new_car.curPoint = trainCars[trainCars.size() - 1].curPoint - trainCars[trainCars.size()-1].length;
	} else {
		new_car.curPoint = (trainCars[trainCars.size() - 1].curPoint - trainCars[trainCars.size()-1].length) + (tPoints.size()-1);
	}
	new_car.carType = BACK_CAR;
	new_car.scale = trainCars[0].scale;
	new_car.getLength();
	trainCars.push_back(new_car);
	//set its velocity
	trainCars[trainCars.size()-1].vel = trainCars[trainCars.size()-2].vel;
	gl_win->damage(1);
}

/*  adds another train to the track behind the last car
*/
void removeCarButtonCallback(Fl_Widget *w){
	if(trainCars.size() > 1) {
		trainCars.pop_back();
		gl_win->damage(1);
	}
}

/* idle call back to handle animation
*/
void idleCallback(void *p){
	if(gl_win->animate){
		for(int i = 0; i < trainCars.size(); i++){
			//move car up velocity points on track
			trainCars[i].curPoint = (trainCars[i].curPoint + trainCars[i].vel)%tPoints.size();
		}
		gl_win->damage(1);
	}
}