/*	Justin Fiedler
	models.cpp
	
	This handles the objects in the world.  Train cars, Trees, and Passengers.  Really
	everything a utopian world could ever need.
*/

#include "models.h"
#include <iostream>
using namespace std;

//external vectors to handle objects
std::vector<Train> trainCars;
std::vector<Tree> trees;

/*	constructs a new train
*/
Train::Train(int _curPoint, float _scale){
	curPoint = _curPoint;
	scale = _scale;
	carType = FRONT_CAR;
	length = 2*(1 - carType)*scale/.01;
	vel = 1;
}


/*	calculates the length of the car in terms of track points
*/
void Train::getLength(){
	length = (2 - carType)*scale/.01;
}

int negMod(int x, int mod){
	if(x < 0){
		return mod + x - 1;
	} else {
		return x%mod;
	}
}

/*	calculates the current rotation of the train car
*/
void Train::getRotation(int arcMode){
	float M_PI = 3.14159265358979323846;
	normal.set3d(0, 1, 0);

	//cout << "tPoint: " << tPoints[curPoint].x << " " << tPoints[curPoint].y << " " << tPoints[curPoint].z << endl;
	
	point dif;
	int front_pos, back_pos;
	switch(arcMode){
		case ARC_LENGTH:
			//	stick to track
			front_pos = (curPoint + (int)(length/2))%tPoints.size();
			back_pos = negMod(curPoint - (int)(length/2), tPoints.size());
			cout << front_pos << " " << back_pos << endl;
			x_vect.x = tPoints[front_pos].x - tPoints[back_pos].x;
			x_vect.y = tPoints[front_pos].y  - tPoints[back_pos].y;
			x_vect.z = tPoints[front_pos].z  - tPoints[back_pos].z;
			
			dif.setp(x_vect);

			z_vect.setp(x_vect.crossProduct(normal));
			z_vect.normalize();	

			y_vect.setp(x_vect.crossProduct(z_vect));
			y_vect.normalize();
			y_vect.x = -y_vect.x;
			y_vect.y = -y_vect.y;
			y_vect.z = -y_vect.z;

			rot.x = atan2(dif.x, dif.z)*180/M_PI;
			rot.y = atan2(dif.y, sqrt(dif.x*dif.x+dif.z*dif.z))*180/M_PI;
			rot.z = 0; //atan2(
			break;
		default:
			x_vect.x = tPoints[(curPoint + 1)%tPoints.size()].x - tPoints[curPoint].x;
			x_vect.y = tPoints[(curPoint + 1)%tPoints.size()].y  - tPoints[curPoint].y;
			x_vect.z = tPoints[(curPoint + 1)%tPoints.size()].z  - tPoints[curPoint].z;
			
			dif.setp(x_vect);

			z_vect.setp(x_vect.crossProduct(normal));
			z_vect.normalize();	

			
			rot.x = atan2(dif.x, dif.z)*180/M_PI;
			rot.y = atan2(dif.y, sqrt(dif.x*dif.x+dif.z*dif.z))*180/M_PI;
			rot.z = 0; //atan2(dif.x, dif.z);
	};
	//cout << "dif: " << dif.x << " " << dif.y << " " << dif.z << endl;
	//cout << "car" << curPoint << " vel: " << vel << " rot:(" << rot.x << ", " << rot.y << ", " << rot.z <<")" << endl << endl;
}

/*	draw the train
*/
void Train::draw(int doColors, int arcMode){
	Passenger pass;
	point p;
	if(arcMode == NON_ARC_LENGTH){
		p.setp(tPoints[curPoint]);
	} else {
		p.setp(tPoints[negMod(curPoint - length/2, tPoints.size())]);
	}
	getRotation(arcMode);
	glPushMatrix();
		glTranslated(p.x, p.y, p.z);
		glRotatef(rot.x, 0, 1, 0);
		glRotatef(-rot.y, 1, 0, 0);
		
		pass.scale = .5*scale;
		pass.p.set3d(0, .5*scale, 0);
		pass.draw(doColors, rot.y < 0?true:false);
		
		if(doColors == 1) glColor3f(0, 0, 3);
		
		switch(carType){
			case FRONT_CAR:
				//body of car
				drawCube(0, scale/2, -scale/2, scale);
				drawCube(0, scale/2, scale/2, scale);
				
				glScalef(scale, scale, scale);
				glBegin(GL_QUADS);
					//front
					if(doColors == 1) glColor3f(1, 0, 0);
					//top
					glNormal3d( 0 , 1 , 1);
					glVertex3d( -.5, 1, 1);
					glVertex3d( -.5, 0, 1.95);
					glVertex3d(  .5, 0, 1.95);
					glVertex3d(  .5, 1, 1);
				glEnd();
		
				glBegin(GL_TRIANGLES);
					if(doColors == 1) glColor3f(1, 0, 0);
					//side 1 triangle
					glNormal3d( 0 , -1 ,0);
					glVertex3d( -.5, 1, 1);
					glVertex3d( -.5, 0, 1.95);
					glVertex3d( -.5, 0, 1);
					
					if(doColors == 1) glColor3f(1, 0, 0);
					//side 2 triangle
					glNormal3d( 0 , 1 ,0);
					glVertex3d( .5, 1, 1);
					glVertex3d( .5, 0, 1.95);
					glVertex3d( .5, 0, 1);
				glEnd();
				break;
			case BACK_CAR:
				drawCube(0, scale/2, 0, scale);
				break;
		};
	glPopMatrix();
}



/*	constructs a new Tree
*/
Tree::Tree(){
	p.set3d(0, 0, 0);
	rotation = 0;
	scale = 1;
}

/*	also constructs a Tree at point <_p> of scale <_scale>
*/
Tree::Tree(point _p, float _scale){
	p.setp(_p);
	rotation = 0;
	scale = _scale;
}


/*	draws the tree in the world
*/
void Tree::draw(int doColors){
	if(doColors == 1) glColor3f(.3, .3, 0);
	drawCube(p.x, p.y + .5*scale, p.z, scale, rotation);
	glPushMatrix();
		glTranslated(p.x, p.y, p.z);
		glRotatef(rotation, 0, 1, 0);
		glScalef(scale, scale, scale);
		glBegin(GL_TRIANGLE_FAN);
			if(doColors == 1) glColor3f(0, 1, 0);
			//side 1
			//glNormal3d( 0 , -1 ,0);
			glVertex3d( 0, 6, 0);
			glVertex3d( -1.5, 1, 1.5);
			glVertex3d( -1.5, 1, -1.5);
			glVertex3d( 1.5, 1, -1.5);
			glVertex3d( 1.5, 1, 1.5);
			glVertex3d( -1.5, 1, 1.5);
		glEnd();
	glPopMatrix();
}

/*	constructs a new passenger
*/
Passenger::Passenger(){
	p.set3d(0, 0, 0);
	rotation = 0;
	scale = 1;
}

/*	constructs a new passenger at point <_p> of scale <_scale>
*/
Passenger::Passenger(point _p, float _scale){
	p.setp(_p);
	rotation = 0;
	scale = _scale;
}

/*	draws the passenger if <armsUp> it draws their arms up
	else draws their arms down
*/
void Passenger::draw(int doColors, bool armsUp){
	if(doColors == 1) glColor3f(1,1,1);
	glLineWidth(2.0f);
	glPushMatrix();
		glTranslated(p.x, p.y, p.z);
		//body
		drawCube(p.x, p.y + .5*scale, p.z, scale);
		// head
		drawCube(p.x, p.y + 1.35*scale, p.z, .5*scale);
		//shoulders
		drawCube(p.x + 1.15*scale , p.y + .75*scale, p.z, .5*scale);
		drawCube(p.x - 1.15*scale , p.y + .75*scale, p.z, .5*scale);
		//draw arms
		glBegin(GL_LINES);
			glVertex3d(p.x + 1.15*scale , p.y + .75*scale, p.z);
			glVertex3d(p.x + 1.15*scale , p.y + armsUp*1.85*scale, p.z + (!armsUp)*1.85*scale);
			glVertex3d(p.x - 1.15*scale , p.y + .75*scale, p.z);
			glVertex3d(p.x - 1.15*scale , p.y + armsUp*1.85*scale, p.z + (!armsUp)*1.85*scale);
		glEnd();
		//top of hands up
		drawCube(p.x + 1.15*scale , p.y + armsUp*1.85*scale, p.z + (!armsUp)*1.85*scale, .5*scale);
		drawCube(p.x - 1.15*scale , p.y + armsUp*1.85*scale, p.z + (!armsUp)*1.85*scale, .5*scale);
	glPopMatrix();
	glLineWidth(1.0f);
}