/*	Justin Fiedler
	models.h
	
	This handles the objects in the world.  Train cars, Trees, and Passengers.  Really
	everything a utopian world could ever need.
*/

#ifndef _MODELS_
#define _MODELS_

#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>

#include "point.h"
#include "curve.h"
#include "MyGlWindow.h"
#include "3DUtils.h"
#include <vector>

//some constants for the type of car (front or back)
const int FRONT_CAR = 0;
const int BACK_CAR	= 1;

//a class to handle Trains
class Train {
	public:
		int curPoint;		//current point that car is at on the track
		float scale;		//the scale of the car
		float length;		//the length of the car in track points
		int carType;		//the type of car, front or back
		point rot;			//the rotation of the car in x and y directions

		point normal;		//normal, set to y unit vector
		point x_vect;		//the new x axis
		point y_vect;		//the new y axis
		point z_vect;		//the new z axis
		
		int vel;			//the speed of the coaster in units of points on the track
		
		//constructor
		Train(int curPoint=0, float _scale=1);
		void getLength();	//claculates the length of the car in track point distances
		void getRotation(int arcMode);	//calculates the current rotation of the train car
		void draw(int doColors, int arcMode); //draws the Train
		
};

//a class to handle Trees
class Tree {
	public:
		point p;		//location of the tree
		float scale;	//the scale factor
		float rotation;	//rotation of the tree
		
		//constructors
		Tree();
		Tree(point _p, float _scale=1);
		void draw(int doColors=1);	//draws the tree
};

//a class to handle Passengers
class Passenger {
	public:
		point p;		//location of the passenger
		float scale;	//the scale passenger
		float rotation;	//rotation of the passenger
		
		//constructors
		Passenger();
		Passenger(point _p, float _scale=1);
		void draw(int doColors=1, bool armsUp=false);	//draws the passenger
};

int negMod(int x, int mod);

//external varibles to handle the objects (train cars/ trees)
extern std::vector<Train> trainCars;
extern std::vector<Tree> trees;
#endif